/* modifier 0 means no modifier */
static int surfuseragent    = 1;  /* Append Surf version to default WebKit user agent */
static char *fulluseragent  = ""; /* Or override the whole user agent string */
static char *scriptfile     = "~/.config/surf/script.js";
static char *styledir       = "~/.config/surf/styles/";
static char *certdir        = "~/.config/surf/certificates/";
static char *cachedir       = "~/.config/surf/cache/";
static char *cookiefile     = "~/.config/surf/cookies.txt";
static char **plugindirs    = (char*[]){
	"~/.config/surf/plugins/",
	LIBPREFIX "/mozilla/plugins/",
	NULL
};

/* Webkit default features */
/* Highest priority value will be used.
 * Default parameters are priority 0
 * Per-uri parameters are priority 1
 * Command parameters are priority 2
 */
static Parameter defconfig[ParameterLast] = {
	/* parameter                    Arg value       priority */
	[AccessMicrophone]    =       { { .i = 0 },     },
	[AccessWebcam]        =       { { .i = 0 },     },
	[Certificate]         =       { { .i = 0 },     },
	[CaretBrowsing]       =       { { .i = 0 },     },
	[CookiePolicies]      =       { { .v = "@Aa" }, },
	[DefaultCharset]      =       { { .v = "UTF-8" }, },
	[DiskCache]           =       { { .i = 1 },     },
	[DNSPrefetch]         =       { { .i = 0 },     },
	[FileURLsCrossAccess] =       { { .i = 0 },     },
	[FontSize]            =       { { .i = 12 },    },
	[FrameFlattening]     =       { { .i = 0 },     },
	[Geolocation]         =       { { .i = 0 },     },
	[HideBackground]      =       { { .i = 0 },     },
	[Inspector]           =       { { .i = 0 },     },
	[Ephemeral]           =       { { .i = 0 },     },
	[Java]                =       { { .i = 0 },     },
	[JavaScript]          =       { { .i = 0 },     },
	[KioskMode]           =       { { .i = 0 },     },
	[LoadImages]          =       { { .i = 0 },     },
	[MediaManualPlay]     =       { { .i = 1 },     },
	[PreferredLanguages]  =       { { .v = (char *[]){ NULL } }, },
	[RunInFullscreen]     =       { { .i = 0 },     },
	[ScrollBars]          =       { { .i = 1 },     },
	[ShowIndicators]      =       { { .i = 1 },     },
	[SiteQuirks]          =       { { .i = 1 },     },
	[SmoothScrolling]     =       { { .i = 0 },     },
	[SpellChecking]       =       { { .i = 0 },     },
	[SpellLanguages]      =       { { .v = ((char *[]){ "en_US", NULL }) }, },
	[StrictTLS]           =       { { .i = 1 },     },
	[Style]               =       { { .i = 1 },     },
	[WebGL]               =       { { .i = 0 },     },
	[ZoomLevel]           =       { { .f = 1.0 },   },
};

static UriParameters uriparams[] = {
	{ "(://|\\.)suckless\\.org(/|$)", {
	  [JavaScript] = { { .i = 0 }, 1 },
	}, },
};

/* default window size: width, height */
static int winsize[] = { 800, 600 };

static WebKitFindOptions findopts = WEBKIT_FIND_OPTIONS_CASE_INSENSITIVE |
                                    WEBKIT_FIND_OPTIONS_WRAP_AROUND;

#define PROMPT_GO   "Go:"
#define PROMPT_FIND "Find:"

/* SETPROP(readprop, setprop, prompt)*/
#define SETPROP(r, s, p) { \
        .v = (const char *[]){ "/bin/sh", "-c", \
             "prop=\"$(printf '%b' \"$(xprop -id $1 "r" " \
             "| sed -e 's/^"r"(UTF8_STRING) = \"\\(.*\\)\"/\\1/' " \
             "      -e 's/\\\\\\(.\\)/\\1/g')\" " \
             "| dmenu -p '"p"' -w $1)\" " \
             "&& xprop -id $1 -f "s" 8u -set "s" \"$prop\"", \
             "surf-setprop", winid, NULL \
        } \
}

/* DOWNLOAD(URI, referer) */
#define DOWNLOAD(u, r) { \
        .v = (const char *[]){ "st", "-e", "/bin/sh", "-c",\
             "cd ~/tmp && curl -g -L -J -O -A \"$1\" -b \"$2\" -c \"$2\"" \
             " -e \"$3\" \"$4\"; read", \
             "surf-download", useragent, cookiefile, r, u, NULL \
        } \
}

/* PLUMB(URI) */
/* This called when some URI which does not begin with "about:",
 * "http://" or "https://" should be opened.
 */
#define PLUMB(u) {\
        .v = (const char *[]){ "/bin/sh", "-c", \
             "xdg-open \"$0\"", u, NULL \
        } \
}

/* PLUMBER(URI) */
#define PLUMBER(u) {\
        .v = (const char *[]){ "/bin/sh", "-c", \
             "plumb \"$(echo $(xprop -id $0 $1 | cut -d '\"' -f2) )\"",\
             winid, u, NULL \
        } \
}

/* VIDEOPLAY(URI) */
#define VIDEOPLAY(u) {\
        .v = (const char *[]){ "/bin/sh", "-c", \
             "mpv --really-quiet \"$0\"", u, NULL \
        } \
}

/* VIDEOPLAYTABBED(URI) */
#define VIDEOPLAYTABBED(u) {\
        .v = (const char *[]){ "/bin/sh", "-c", "-x", \
             "mpv --really-quiet \"$(echo $(xprop -id $0 $1 | \
			 cut -d '\"' -f2) )\"",\
             winid, u, NULL \
        } \
}

/* BOOKMARK(URI) */
#define BOOKMARKURI(r) {\
        .v = (const char *[]){ "st", "-e", "/bin/sh", "-c", "-x", \
            "bookmark \"$(echo $(xprop -id $0 $1) | cut -d '\"' -f2)\" " \
            "\"$(echo $(xprop -id $0 _NET_WM_NAME) | cut -d '\"' -f2 " \
            "| cut -d ' | ' -f3- )\" ",\
            winid, r, NULL \
        } \
}

/* FIREFOX(URI) */
#define FIREFOX(u) {\
        .v = (const char *[]){ "/bin/sh", "-c", \
             "firefox \"$(echo $(xprop -id $0 $1 | cut -d '\"' -f2) )\"",\
             winid, u, NULL \
        } \
}

/* EDITSRC(URI) */
#define EDITSRC(r) {\
        .v = (const char *[]){ "st", "-e", "/bin/sh", "-x", "-c", \
			"f=\"$(mktemp)\" && " \
			"curl -sL \"$(echo $(xprop -id $0 $1) | cut -d '\"' -f2)\" > \"$f\" && " \
			"tmux new-session \"$EDITOR '$f'\" ", \
            winid, r, NULL \
        } \
}

/* styles */
/*
 * The iteration will stop at the first match, beginning at the beginning of
 * the list.
 */
static SiteSpecific styles[] = {
	/* regexp               file in $styledir */
	{ ".*",                 "default.css" },
};

/* certificates */
/*
 * Provide custom certificate for urls
 */
static SiteSpecific certs[] = {
	/* regexp               file in $certdir */
	{ "://suckless\\.org/", "suckless.org.crt" },
};

#define MODKEY GDK_CONTROL_MASK

/* hotkeys */
/*
 * If you use anything else but MODKEY and GDK_SHIFT_MASK, don't forget to
 * edit the CLEANMASK() macro.
 */
static Key keys[] = {
	/* modifier              keyval          function    arg */
	{ MODKEY,                GDK_KEY_slash,  spawn,      SETPROP("_SURF_FIND", "_SURF_FIND", PROMPT_FIND) },
	{ 0,                     GDK_KEY_Escape, stop,       { 0 } },
	{ MODKEY,                GDK_KEY_space,  scrollv,    { .i = +50 } },
	{ MODKEY|GDK_SHIFT_MASK, XK_Next,        zoom,       { .i = -1 } },
	{ MODKEY|GDK_SHIFT_MASK, XK_Prior,       zoom,       { .i = +1 } },
	{ MODKEY|GDK_SHIFT_MASK, XK_Home,        zoom,       { .i = 0  } },

	{ MODKEY,                GDK_KEY_F11,    togglefullscreen, { 0 } },

	{ MODKEY|GDK_SHIFT_MASK, GDK_KEY_a,      togglecookiepolicy, { 0 } },
	{ MODKEY,                GDK_KEY_b,      spawn,      BOOKMARKURI("_SURF_URI") },
	{ MODKEY|GDK_SHIFT_MASK, GDK_KEY_b,      toggle,     { .i = ScrollBars } },
	{ MODKEY,                GDK_KEY_c,      stop,       { 0 } },
	{ MODKEY|GDK_SHIFT_MASK, GDK_KEY_c,      toggle,     { .i = CaretBrowsing } },
	{ MODKEY,                GDK_KEY_d,      scrollv,    { .i = +50 } },
	{ MODKEY|GDK_SHIFT_MASK, GDK_KEY_d,      scrollv,    { .i = +100 } },
	{ MODKEY,                GDK_KEY_e,      spawn,      EDITSRC("_SURF_URI") },
	{ MODKEY,                GDK_KEY_f,      spawn,      FIREFOX("_SURF_URI") },
	{ MODKEY|GDK_SHIFT_MASK, GDK_KEY_f,      toggle,     { .i = FrameFlattening } },
	{ MODKEY,                GDK_KEY_h,      navigate,   { .i = -1 } },
	{ MODKEY,                GDK_KEY_i,      scrollh,    { .i = +10 } },
	{ MODKEY|GDK_SHIFT_MASK, GDK_KEY_i,      toggle,     { .i = LoadImages } },
	{ MODKEY,                GDK_KEY_j,      scrollv,    { .i = +10 } },
	{ MODKEY,                GDK_KEY_k,      scrollv,    { .i = -10 } },
	{ MODKEY,                GDK_KEY_l,      navigate,   { .i = +1 } },
	{ MODKEY|GDK_SHIFT_MASK, GDK_KEY_l,      toggle,     { .i = Geolocation } },
	{ MODKEY,                GDK_KEY_m,      spawn,      VIDEOPLAYTABBED("_SURF_URI") },
	{ MODKEY|GDK_SHIFT_MASK, GDK_KEY_m,      toggle,     { .i = Style } },
	{ MODKEY,                GDK_KEY_n,      find,       { .i = +1 } },
	{ MODKEY|GDK_SHIFT_MASK, GDK_KEY_n,      find,       { .i = -1 } },
	{ MODKEY,                GDK_KEY_o,      spawn,      SETPROP("_SURF_URI", "_SURF_GO", PROMPT_GO) },
	{ MODKEY|GDK_SHIFT_MASK, GDK_KEY_o,      toggleinspector, { 0 } },
	{ MODKEY,                GDK_KEY_p,      clipboard,  { .i = 1 } },
	{ MODKEY|GDK_SHIFT_MASK, GDK_KEY_p,      print,      { 0 } },
	{ MODKEY|GDK_SHIFT_MASK, GDK_KEY_r,      reload,     { .i = 1 } },
	{ MODKEY,                GDK_KEY_r,      reload,     { .i = 0 } },
	{ MODKEY|GDK_SHIFT_MASK, GDK_KEY_s,      toggle,     { .i = JavaScript } },
	{ MODKEY,                GDK_KEY_t,      showcert,   { 0 } },
	{ MODKEY|GDK_SHIFT_MASK, GDK_KEY_t,      toggle,     { .i = StrictTLS } },
	{ MODKEY,                GDK_KEY_u,      scrollv,    { .i = -50 } },
	{ MODKEY|GDK_SHIFT_MASK, GDK_KEY_u,      spawn,      PLUMBER("_SURF_URI") },
	{ MODKEY,                GDK_KEY_y,      clipboard,  { .i = 0 } },
};

/* button definitions */
/* target can be OnDoc, OnLink, OnImg, OnMedia, OnEdit, OnBar, OnSel, OnAny */
static Button buttons[] = {
	/* target       event mask      button  function        argument        stop event */
	{ OnLink,       0,              2,      clicknewwindow, { .i = 0 },     1 },
	{ OnLink,       MODKEY,         2,      clicknewwindow, { .i = 1 },     1 },
	{ OnLink,       MODKEY,         1,      clicknewwindow, { .i = 1 },     1 },
	{ OnAny,        0,              8,      clicknavigate,  { .i = -1 },    1 },
	{ OnAny,        0,              9,      clicknavigate,  { .i = +1 },    1 },
	{ OnMedia,      MODKEY,         1,      clickexternplayer, { 0 },       1 },
};
